@@ -34,6 +34,7 @@
 
 #include <config.h>
 
+#include <stdbool.h>
 #include <sys/poll.h>
 #include <unistd.h>
 #include <string.h>
@@ -51,6 +52,7 @@
 #include <infiniband/umad.h>
 
 #define IB_OPENIB_OUI                 (0x001405)
+#define CAPMASK_IS_SM_DISABLED        (0x400)
 
 #include <valgrind/memcheck.h>
 #include "sysfs.h"
@@ -78,6 +80,16 @@ struct ib_user_mad_reg_req2 {
 	uint8_t  reserved[3];
 };
 
+struct port_guid_port_count {
+	__be64 port_guid;
+	uint8_t   count;
+};
+
+struct guid_ca_pairs_mapping {
+	__be64 port_guid;
+	struct umad_ca_pair *ca_pair;
+};
+
 #define IBWARN(fmt, args...) fprintf(stderr, "ibwarn: [%d] %s: " fmt "\n", getpid(), __func__, ## args)
 
 #define TRACE	if (umaddebug)	IBWARN
@@ -128,6 +140,11 @@ static int put_ca(umad_ca_t * ca)
 	return 0;		/* caching not implemented yet */
 }
 
+static unsigned is_smi_disabled(umad_port_t *port)
+{
+	return (be32toh(port->capmask) & CAPMASK_IS_SM_DISABLED);
+}
+
 static int release_port(umad_port_t * port)
 {
 	free(port->pkeys);
@@ -248,8 +265,9 @@ static int release_ca(umad_ca_t * ca)
  * the first port that is active, and if such is not found, to
  * the first port that is link up and if none are linkup, then
  * the first port that is not disabled.  Otherwise return -1.
+ * if enforce_smi > 0, only search smi ports. if none are found, return -1.
  */
-static int resolve_ca_port(const char *ca_name, int *port)
+static int resolve_ca_port(const char *ca_name, int *port, unsigned enforce_smi)
 {
 	umad_ca_t ca;
 	int active = -1, up = -1;
@@ -280,6 +298,10 @@ static int resolve_ca_port(const char *ca_name, int *port)
 			ret = -1;
 			goto Exit;
 		}
+		if (enforce_smi && is_smi_disabled(ca.ports[*port])) {
+			ret = -1;
+			goto Exit;
+		}
 		if (ca.ports[*port]->state == 4) {
 			ret = 1;
 			goto Exit;
@@ -297,6 +319,8 @@ static int resolve_ca_port(const char *ca_name, int *port)
 		if (strcmp(ca.ports[i]->link_layer, "InfiniBand") &&
 		    strcmp(ca.ports[i]->link_layer, "IB"))
 			continue;
+		if (enforce_smi && is_smi_disabled(ca.ports[i]))
+			continue;
 		if (up < 0 && ca.ports[i]->phys_state == 5)
 			up = *port = i;
 		if (ca.ports[i]->state == 4) {
@@ -311,6 +335,8 @@ static int resolve_ca_port(const char *ca_name, int *port)
 			DEBUG("checking port %d", i);
 			if (!ca.ports[i])
 				continue;
+			if (enforce_smi && is_smi_disabled(ca.ports[i]))
+				continue;
 			if (ca.ports[i]->phys_state != 3) {
 				up = *port = i;
 				break;
@@ -333,7 +359,7 @@ Exit:
 }
 
 static int resolve_ca_name(const char *ca_in, int *best_port,
-			   char **ca_name)
+			   char **ca_name, unsigned enforce_smi)
 {
 	struct umad_device_node *device_list;
 	struct umad_device_node *node;
@@ -350,7 +376,7 @@ static int resolve_ca_name(const char *ca_in, int *best_port,
 	}
 
 	if (ca_in) {
-		if (resolve_ca_port(ca_in, best_port) < 0)
+		if (resolve_ca_port(ca_in, best_port, enforce_smi) < 0)
 			return -1;
 		*ca_name = strdup(ca_in);
 		if (!(*ca_name))
@@ -370,7 +396,7 @@ static int resolve_ca_name(const char *ca_in, int *best_port,
 		TRACE("checking ca '%s'", name_found);
 
 		port = best_port ? *best_port : 0;
-		port_type = resolve_ca_port(name_found, &port);
+		port_type = resolve_ca_port(name_found, &port, enforce_smi);
 		if (port_type < 0)
 			continue;
 
@@ -626,7 +652,7 @@ int umad_get_ca_portguids(const char *ca_name, __be64 *portguids, int max)
 	char *found_ca_name;
 
 	TRACE("ca name %s max port guids %d", ca_name, max);
-	if (resolve_ca_name(ca_name, NULL, &found_ca_name) < 0) {
+	if (resolve_ca_name(ca_name, NULL, &found_ca_name, 0) < 0) {
 		result = -ENODEV;
 		goto exit;
 	}
@@ -665,7 +691,7 @@ int umad_get_issm_path(const char *ca_name, int portnum, char path[], int max)
 
 	TRACE("ca %s port %d", ca_name, portnum);
 
-	if (resolve_ca_name(ca_name, &portnum, &found_ca_name) < 0) {
+	if (resolve_ca_name(ca_name, &portnum, &found_ca_name, 0) < 0) {
 		result = -ENODEV;
 		goto exit;
 	}
@@ -685,7 +711,7 @@ exit:
 	return result;
 }
 
-int umad_open_port(const char *ca_name, int portnum)
+static int do_umad_open_port(const char *ca_name, int portnum, unsigned enforce_smi)
 {
 	char dev_file[UMAD_DEV_FILE_SZ];
 	int umad_id, fd, result;
@@ -699,7 +725,7 @@ int umad_open_port(const char *ca_name, int portnum)
 		goto exit;
 	}
 
-	if (resolve_ca_name(ca_name, &portnum, &found_ca_name) < 0) {
+	if (resolve_ca_name(ca_name, &portnum, &found_ca_name, enforce_smi) < 0) {
 		result = -ENODEV;
 		goto exit;
 	}
@@ -735,13 +761,23 @@ exit:
 	return result;
 }
 
+int umad_open_port(const char *ca_name, int portnum)
+{
+	return do_umad_open_port(ca_name, portnum, 0);
+}
+
+int umad_open_smi_port(const char *ca_name, int portnum)
+{
+	return do_umad_open_port(ca_name, portnum, 1);
+}
+
 int umad_get_ca(const char *ca_name, umad_ca_t *ca)
 {
 	int r = 0;
 	char *found_ca_name;
 
 	TRACE("ca_name %s", ca_name);
-	if (resolve_ca_name(ca_name, NULL, &found_ca_name) < 0) {
+	if (resolve_ca_name(ca_name, NULL, &found_ca_name, 0) < 0) {
 		r = -ENODEV;
 		goto exit;
 	}
@@ -783,7 +819,7 @@ int umad_get_port(const char *ca_name, int portnum, umad_port_t *port)
 
 	TRACE("ca_name %s portnum %d", ca_name, portnum);
 
-	if (resolve_ca_name(ca_name, &portnum, &found_ca_name) < 0) {
+	if (resolve_ca_name(ca_name, &portnum, &found_ca_name, 0) < 0) {
 		result = -ENODEV;
 		goto exit;
 	}
@@ -1003,7 +1039,7 @@ int umad_get_fd(int fd)
 }
 
 int umad_register_oui(int fd, int mgmt_class, uint8_t rmpp_version,
-		      uint8_t oui[3], long method_mask[])
+		      uint8_t oui[3], long method_mask[16 / sizeof(long)])
 {
 	struct ib_user_mad_reg_req req;
 
@@ -1042,7 +1078,7 @@ int umad_register_oui(int fd, int mgmt_class, uint8_t rmpp_version,
 }
 
 int umad_register(int fd, int mgmt_class, int mgmt_version,
-		  uint8_t rmpp_version, long method_mask[])
+		  uint8_t rmpp_version, long method_mask[16 / sizeof(long)])
 {
 	struct ib_user_mad_reg_req req;
 	__be32 oui = htobe32(IB_OPENIB_OUI);
@@ -1329,3 +1365,301 @@ void umad_free_ca_device_list(struct umad_device_node *head)
 		free(node);
 	}
 }
+
+static struct umad_ca_pair *get_ca_pair_from_arr_by_guid(__be64 port_guid,
+					struct guid_ca_pairs_mapping mapping[],
+					size_t map_max, size_t *map_added,
+					struct umad_ca_pair devs[],
+					size_t devs_max, size_t *devs_added)
+{
+	struct umad_ca_pair *dev = NULL;
+	// attempt to find the port guid in the mapping
+	size_t i = 0;
+
+	for (i = 0; i < *map_added; ++i) {
+		if (mapping[i].port_guid == port_guid)
+			return mapping[i].ca_pair;
+	}
+
+	// attempt to add a new mapping/device
+	if (*map_added >= map_max || *devs_added >= devs_max)
+		return NULL;
+
+	dev = &devs[*devs_added];
+	mapping[*map_added].port_guid = port_guid;
+	mapping[*map_added].ca_pair = dev;
+	(*devs_added)++;
+	(*map_added)++;
+
+	return dev;
+}
+
+static uint8_t get_port_guid_count(__be64 guid, const struct port_guid_port_count counts[],
+							size_t max_guids)
+{
+	size_t i = 0;
+
+	for (i = 0; i < max_guids; ++i) {
+		if (counts[i].port_guid == guid)
+			return counts[i].count;
+	}
+
+	return 0;
+}
+
+
+static bool find_port_guid_count(struct port_guid_port_count counts[], size_t max,
+						  __be64 port_guid, size_t *index)
+{
+	size_t i = 0;
+
+	for (i = 0; i < max; ++i) {
+		if (counts[i].port_guid == 0) {
+			*index = i;
+			return false;
+		}
+		if (counts[i].port_guid == port_guid) {
+			*index = i;
+			return true;
+		}
+	}
+
+	*index = max;
+	return false;
+}
+
+static int count_ports_by_guid(char legacy_ca_names[][UMAD_CA_NAME_LEN], size_t num_cas,
+						struct port_guid_port_count counts[], size_t max)
+{
+	// how many unique port GUIDs were added
+	size_t num_of_guid = 0;
+
+	memset(counts, 0, max * sizeof(struct port_guid_port_count));
+
+	size_t c_idx = 0;
+
+	for (c_idx = 0; c_idx < num_cas; ++c_idx) {
+		umad_ca_t curr_ca;
+
+		if (umad_get_ca(legacy_ca_names[c_idx], &curr_ca) < 0)
+			continue;
+
+		size_t p_idx = 1;
+
+		for (p_idx = 1; p_idx < (size_t)curr_ca.numports + 1; ++p_idx) {
+			umad_port_t *p_port = curr_ca.ports[p_idx];
+			size_t count_idx = 0;
+
+			if (!p_port)
+				continue;
+
+			if (find_port_guid_count(counts, max, p_port->port_guid, &count_idx)) {
+				// port GUID already has a count struct
+				++counts[count_idx].count;
+			} else {
+				// add a new count struct for this GUID.
+				// if the maximum amount was already added, do nothing.
+				if (count_idx != max) {
+					counts[count_idx].port_guid = p_port->port_guid;
+					counts[count_idx].count = 1;
+					++num_of_guid;
+				}
+			}
+		}
+
+		umad_release_ca(&curr_ca);
+	}
+
+	return num_of_guid;
+}
+
+int umad_get_smi_gsi_pairs(struct umad_ca_pair cas[], size_t max)
+{
+	size_t added_devices = 0, added_mappings = 0;
+	char legacy_ca_names[UMAD_MAX_DEVICES][UMAD_CA_NAME_LEN] = {};
+	struct port_guid_port_count counts[UMAD_MAX_PORTS] = {};
+	struct guid_ca_pairs_mapping mapping[UMAD_MAX_PORTS] = {};
+
+	memset(cas, 0, sizeof(struct umad_ca_pair) * max);
+	int cas_found = umad_get_cas_names(legacy_ca_names, UMAD_MAX_DEVICES);
+
+	if (cas_found < 0)
+		return 0;
+
+	count_ports_by_guid(legacy_ca_names, cas_found, counts, UMAD_MAX_PORTS);
+
+	size_t c_idx = 0;
+
+	for (c_idx = 0; c_idx < (size_t)cas_found; ++c_idx) {
+		umad_ca_t curr_ca;
+
+		if (umad_get_ca(legacy_ca_names[c_idx], &curr_ca) < 0)
+			continue;
+
+		size_t p_idx = 1;
+
+		for (p_idx = 1; p_idx < (size_t)curr_ca.numports + 1; ++p_idx) {
+			umad_port_t *p_port = curr_ca.ports[p_idx];
+			uint8_t guid_count = 0;
+
+			if (!p_port)
+				continue;
+
+			guid_count = get_port_guid_count(curr_ca.ports[p_idx]->port_guid,
+								counts, UMAD_MAX_PORTS);
+			struct umad_ca_pair *dev = get_ca_pair_from_arr_by_guid(p_port->port_guid,
+								mapping, UMAD_MAX_PORTS,
+								&added_mappings, cas,
+								max, &added_devices);
+				if (!dev)
+					continue;
+			if (guid_count > 1) {
+				// planarized port
+				char *dev_name = is_smi_disabled(p_port) ? dev->gsi_name : dev->smi_name;
+				strncpy(dev_name, curr_ca.ca_name, UMAD_CA_NAME_LEN);
+				break;
+			} else if (guid_count == 1) {
+				if (!is_smi_disabled(p_port))
+					strncpy(dev->smi_name, curr_ca.ca_name, UMAD_CA_NAME_LEN);
+				strncpy(dev->gsi_name, curr_ca.ca_name, UMAD_CA_NAME_LEN);
+				break;
+			} else {
+				umad_release_ca(&curr_ca);
+				return -1;
+			}
+		}
+
+		umad_release_ca(&curr_ca);
+	}
+
+	return added_devices;
+}
+
+static int umad_check_active(const umad_ca_t *ca, int prefered_portnum)
+{
+	if (!ca)
+		return 1;
+
+	if (!ca->ports[prefered_portnum])
+		return 1;
+
+	int state = ca->ports[prefered_portnum]->state;
+
+	return !(state > 1);
+}
+
+static int umad_find_active(struct umad_ca_pair *ca_pair, const umad_ca_t *ca, bool is_gsi)
+{
+	size_t i = 1;
+	uint32_t *portnum_to_set = is_gsi ?
+				&ca_pair->gsi_preferred_port :
+				&ca_pair->smi_preferred_port;
+
+	if (!ca_pair)
+		return 1;
+
+	for (i = 1; i < (size_t)ca->numports + 1; ++i) {
+		if (!umad_check_active(ca, i)) {
+			*portnum_to_set = ca->ports[i]->portnum;
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
+static int find_preferred_ports(struct umad_ca_pair *ca_pair, const umad_ca_t *ca, bool is_gsi, int portnum)
+{
+	if (portnum) {
+		//in case we have same device, use same port for smi/gsi
+		if (!strncmp(ca_pair->gsi_name, ca_pair->smi_name, UMAD_CA_NAME_LEN)) {
+			if (!umad_check_active(ca, portnum)) {
+				ca_pair->gsi_preferred_port = portnum;
+				ca_pair->smi_preferred_port = portnum;
+				return 0;
+			}
+			return 1;
+		}
+		uint32_t *port_to_set = is_gsi ?
+					&ca_pair->gsi_preferred_port :
+					&ca_pair->smi_preferred_port;
+		if (!umad_check_active(ca, portnum)) {
+			*port_to_set = portnum;
+			return umad_find_active(ca_pair, ca, !is_gsi);
+		}
+		return 1;
+	}
+
+	return umad_find_active(ca_pair, ca, false)
+			  + umad_find_active(ca_pair, ca, true);
+
+}
+
+int umad_get_smi_gsi_pair_by_ca_name(const char *name, uint8_t portnum, struct umad_ca_pair *ca_pair,
+				     unsigned enforce_smi)
+{
+	int rc			= 1;
+	size_t i		= 0;
+	int num_cas		= 0;
+	bool is_gsi		= false;
+
+	umad_ca_t ca;
+	struct umad_ca_pair cas_pair[UMAD_MAX_PORTS] = {};
+
+	if (!ca_pair)
+		return -1;
+
+	memset(cas_pair, 0, sizeof(cas_pair));
+	memset(ca_pair, 0, sizeof(*ca_pair));
+
+	num_cas = umad_get_smi_gsi_pairs(cas_pair, UMAD_MAX_PORTS);
+
+	if (num_cas <= 0)
+		return num_cas;
+
+	for (i = 0; i < (size_t)num_cas; ++i) {
+		if (enforce_smi && !cas_pair[i].smi_name[0])
+			continue;
+		if (!cas_pair[i].gsi_name[0])
+			continue;
+
+		if (name)
+			// name doesn't match - keep searching
+			if (strncmp(cas_pair[i].gsi_name, name, UMAD_CA_NAME_LEN)
+				&& strncmp(cas_pair[i].smi_name, name, UMAD_CA_NAME_LEN)) {
+				continue;
+			}
+
+		// check that the device given by "name" has a port number "portnum"
+		// (if name doesn't exist, assume SMI port is given)
+		// (if enforce_smi is false assume GSI in case no SMI present)
+		is_gsi = (!enforce_smi && !cas_pair[i].smi_name[0]) ||
+		(name && !strncmp(name, cas_pair[i].gsi_name, UMAD_CA_NAME_LEN));
+
+		if (umad_get_ca(is_gsi ? cas_pair[i].gsi_name : cas_pair[i].smi_name, &ca) < 0)
+			continue;
+
+		if (portnum) {
+			if (!ca.ports[portnum]) {
+				umad_release_ca(&ca);
+				continue;
+			}
+		}
+
+		// fill candidate
+		*ca_pair = cas_pair[i];
+		rc = find_preferred_ports(ca_pair, &ca, is_gsi, portnum);
+		umad_release_ca(&ca);
+
+		if (!rc)
+			break;
+	}
+
+	if (rc) {
+		errno = ENODEV;
+		return -errno;
+	}
+
+	return rc;
+}
+
