@@ -98,6 +98,7 @@ int ibv_cmd_query_port(struct ibv_context *context, uint8_t port_num,
 		copy_query_port_resp_to_port_attr(port_attr,
 						  &resp_ex.legacy_resp);
 		port_attr->port_cap_flags2 = resp_ex.port_cap_flags2;
+		port_attr->active_speed_ex = resp_ex.active_speed_ex;
 		break;
 	default:
 		return ret;
@@ -666,6 +667,15 @@ int ibv_cmd_query_device_any(struct ibv_context *context,
 
 	if (CAN_COPY(xrc_odp_caps, xrc_odp_caps))
 		attr->xrc_odp_caps = resp->xrc_odp_caps;
+
+	if (attr_size >= offsetofend(struct ibv_device_attr_ex, phys_port_cnt_ex)) {
+		struct verbs_sysfs_dev *sysfs_dev = verbs_get_device(context->device)->sysfs;
+
+		if (sysfs_dev->num_ports)
+			attr->phys_port_cnt_ex = sysfs_dev->num_ports;
+		else
+			attr->phys_port_cnt_ex = attr->orig_attr.phys_port_cnt;
+	}
 #undef CAN_COPY
 
 	return 0;
