@@ -556,7 +556,8 @@ int ibv_cmd_advise_mr(struct ibv_pd *pd,
 		      uint32_t num_sge);
 int ibv_cmd_reg_dmabuf_mr(struct ibv_pd *pd, uint64_t offset, size_t length,
 			  uint64_t iova, int fd, int access,
-			  struct verbs_mr *vmr);
+			  struct verbs_mr *vmr,
+			  struct ibv_command_buffer *driver);
 int ibv_cmd_alloc_mw(struct ibv_pd *pd, enum ibv_mw_type type,
 		     struct ibv_mw *mw, struct ibv_alloc_mw *cmd,
 		     size_t cmd_size,
