@@ -41,6 +41,27 @@
 #define __MEMORY_H__
 
 #include <pthread.h>
+#include "main.h"
+
+struct bnxt_re_mem {
+	void *va_head;
+	void *va_tail;
+	uint32_t head;
+	uint32_t tail;
+	uint32_t size;
+	uint32_t pad;
+};
+
+#define BNXT_RE_QATTR_SQ_INDX	0
+#define BNXT_RE_QATTR_RQ_INDX	1
+struct bnxt_re_qattr {
+	uint32_t esize;
+	uint32_t slots;
+	uint32_t nwr;
+	uint32_t sz_ring;
+	uint32_t sz_shad;
+	uint32_t sw_nwr;
+};
 
 struct bnxt_re_queue {
 	void *va;
@@ -66,10 +87,16 @@ struct bnxt_re_queue {
 	pthread_spinlock_t qlock;
 	uint32_t msn;
 	uint32_t msn_tbl_sz;
+	/*
+	 * This flag is set when CQ is resized. It will be cleared after the
+	 * first CQE is received on the newly resized CQ
+	 */
+	bool cq_resized;
+
+	/* this tracks the 'head' index on the old CQ before resizing */
+	uint32_t old_head;
 };
 
-int bnxt_re_alloc_aligned(struct bnxt_re_queue *que, uint32_t pg_size);
-void bnxt_re_free_aligned(struct bnxt_re_queue *que);
 
 /* Basic queue operation */
 static inline void *bnxt_re_get_hwqe(struct bnxt_re_queue *que, uint32_t idx)
@@ -123,4 +150,9 @@ static inline void bnxt_re_incr_head(struct bnxt_re_queue *que, uint8_t cnt)
 	}
 }
 
+void bnxt_re_free_mem(struct bnxt_re_mem *mem);
+void *bnxt_re_alloc_mem(size_t size, uint32_t pg_size);
+void *bnxt_re_get_obj(struct bnxt_re_mem *mem, size_t req);
+void *bnxt_re_get_ring(struct bnxt_re_mem *mem, size_t req);
+
 #endif
