@@ -132,7 +132,7 @@ struct ibv_pd *qelr_alloc_pd(struct ibv_context *context)
 
 	pd->pd_id = resp.pd_id;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_INIT, "Allocated pd: %d\n", pd->pd_id);
+	verbs_debug(&cxt->ibv_ctx, "Allocated pd: %d\n", pd->pd_id);
 
 	return &pd->ibv_pd;
 }
@@ -143,7 +143,7 @@ int qelr_dealloc_pd(struct ibv_pd *ibpd)
 	struct qelr_pd *pd = get_qelr_pd(ibpd);
 	struct qelr_devctx *cxt = get_qelr_ctx(ibpd->context);
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_INIT, "Deallocated pd: %d\n",
+	verbs_debug(&cxt->ibv_ctx, "Deallocated pd: %d\n",
 		   pd->pd_id);
 
 	rc = ibv_cmd_dealloc_pd(ibpd);
@@ -177,7 +177,7 @@ struct ibv_mr *qelr_reg_mr(struct ibv_pd *ibpd, void *addr, size_t len,
 		return NULL;
 	}
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_MR,
+	verbs_debug(&cxt->ibv_ctx,
 		   "MR Register %p completed successfully pd_id=%d addr=%p len=%zu access=%d lkey=%x rkey=%x\n",
 		   mr, pd->pd_id, addr, len, access, mr->vmr.ibv_mr.lkey,
 		   mr->vmr.ibv_mr.rkey);
@@ -194,7 +194,7 @@ int qelr_dereg_mr(struct verbs_mr *vmr)
 	if (rc)
 		return rc;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_MR,
+	verbs_debug(&cxt->ibv_ctx,
 		   "MR DERegister %p completed successfully\n", vmr);
 
 	free(vmr);
@@ -227,12 +227,12 @@ struct ibv_cq *qelr_create_cq(struct ibv_context *context, int cqe,
 	int chain_size;
 	int rc;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_CQ,
+	verbs_debug(&cxt->ibv_ctx,
 		   "create cq: context=%p, cqe=%d, channel=%p, comp_vector=%d\n",
 		   context, cqe, channel, comp_vector);
 
 	if (!cqe || cqe > cxt->max_cqes) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "create cq: failed. attempted to allocate %d cqes but valid range is 1...%d\n",
 		       cqe, cxt->max_cqes);
 		errno = EINVAL;
@@ -257,7 +257,7 @@ struct ibv_cq *qelr_create_cq(struct ibv_context *context, int cqe,
 			       &cq->ibv_cq, &cmd.ibv_cmd, sizeof(cmd),
 			       &resp.ibv_resp, sizeof(resp));
 	if (rc) {
-		DP_ERR(cxt->dbg_fp, "create cq: failed with rc = %d\n", rc);
+		verbs_err(&cxt->ibv_ctx, "create cq: failed with rc = %d\n", rc);
 		goto err_1;
 	}
 
@@ -274,7 +274,7 @@ struct ibv_cq *qelr_create_cq(struct ibv_context *context, int cqe,
 		if (cq->db_rec_map == MAP_FAILED) {
 			int errsv = errno;
 
-			DP_ERR(cxt->dbg_fp,
+			verbs_err(&cxt->ibv_ctx,
 			       "alloc context: doorbell rec mapping failed resp.db_rec_addr = %llx size=%d context->cmd_fd=%d errno=%d\n",
 			       resp.db_rec_addr, cxt->kernel_page_size,
 			       context->cmd_fd, errsv);
@@ -294,7 +294,7 @@ struct ibv_cq *qelr_create_cq(struct ibv_context *context, int cqe,
 	cq->latest_cqe = NULL; /* must be different from chain_toggle */
 	consume_cqe(cq);
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_CQ,
+	verbs_debug(&cxt->ibv_ctx,
 		   "create cq: successfully created %p\n", cq);
 
 	return &cq->ibv_cq;
@@ -313,11 +313,11 @@ int qelr_destroy_cq(struct ibv_cq *ibv_cq)
 	struct qelr_cq *cq = get_qelr_cq(ibv_cq);
 	int rc;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_CQ, "destroy cq: %p\n", cq);
+	verbs_debug(&cxt->ibv_ctx, "destroy cq: %p\n", cq);
 
 	rc = ibv_cmd_destroy_cq(ibv_cq);
 	if (rc) {
-		DP_VERBOSE(cxt->dbg_fp, QELR_MSG_CQ,
+		verbs_debug(&cxt->ibv_ctx,
 		           "destroy cq: failed to destroy %p, got %d.\n", cq,
 			   rc);
 		return rc;
@@ -327,7 +327,7 @@ int qelr_destroy_cq(struct ibv_cq *ibv_cq)
 	if (cq->db_rec_map)
 		munmap(cq->db_rec_map, cxt->kernel_page_size);
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_CQ,
+	verbs_debug(&cxt->ibv_ctx,
 		   "destroy cq: successfully destroyed %p\n", cq);
 
 	free(cq);
@@ -338,7 +338,7 @@ int qelr_destroy_cq(struct ibv_cq *ibv_cq)
 static struct qelr_srq *qelr_get_srq(struct qelr_devctx *cxt, uint32_t srq_id)
 {
 	if (unlikely(srq_id >= QELR_MAX_SRQ_ID)) {
-		DP_ERR(cxt->dbg_fp, "invalid srq_id %u\n", srq_id);
+		verbs_err(&cxt->ibv_ctx, "invalid srq_id %u\n", srq_id);
 		return NULL;
 	}
 
@@ -430,7 +430,7 @@ static int qelr_create_srq_buffers(struct qelr_devctx *cxt,
 	rc = qelr_chain_alloc(&srq->hw_srq.chain, chain_size,
 			      cxt->kernel_page_size, QELR_RQE_ELEMENT_SIZE);
 	if (rc) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "create srq: failed to map srq, got %d", rc);
 		return rc;
 	}
@@ -440,7 +440,7 @@ static int qelr_create_srq_buffers(struct qelr_devctx *cxt,
 		    MAP_PRIVATE | MAP_ANONYMOUS, -1,
 		    0);
 	if (addr == MAP_FAILED) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "create srq: failed to map producer, got %d", errno);
 		qelr_chain_free(&srq->hw_srq.chain);
 		return errno;
@@ -544,7 +544,7 @@ static inline int qelr_create_qp_buffers_sq(struct qelr_devctx *cxt,
 	rc = qelr_chain_alloc(&qp->sq.chain, chain_size, cxt->kernel_page_size,
 			      QELR_SQE_ELEMENT_SIZE);
 	if (rc)
-		DP_ERR(cxt->dbg_fp, "create qp: failed to map SQ chain, got %d", rc);
+		verbs_err(&cxt->ibv_ctx, "create qp: failed to map SQ chain, got %d", rc);
 
 	qp->sq.max_wr = max_send_wr;
 	qp->sq.max_sges = cxt->sges_per_send_wr;
@@ -571,7 +571,7 @@ static inline int qelr_create_qp_buffers_rq(struct qelr_devctx *cxt,
 	rc = qelr_chain_alloc(&qp->rq.chain, chain_size, cxt->kernel_page_size,
 			      QELR_RQE_ELEMENT_SIZE);
 	if (rc)
-		DP_ERR(cxt->dbg_fp, "create qp: failed to map RQ chain, got %d", rc);
+		verbs_err(&cxt->ibv_ctx, "create qp: failed to map RQ chain, got %d", rc);
 
 	qp->rq.max_wr = max_recv_wr;
 	qp->rq.max_sges = cxt->sges_per_recv_wr;
@@ -624,7 +624,7 @@ static inline int qelr_configure_qp_sq(struct qelr_devctx *cxt,
 		if (qp->sq.db_rec_map == MAP_FAILED) {
 			int errsv = errno;
 
-			DP_ERR(cxt->dbg_fp,
+			verbs_err(&cxt->ibv_ctx,
 			       "alloc context: doorbell rec mapping failed resp.db_rec_addr = %llx size=%d context->cmd_fd=%d errno=%d\n",
 			       resp->sq_db_rec_addr, cxt->kernel_page_size,
 			       cxt->ibv_ctx.context.cmd_fd, errsv);
@@ -642,7 +642,7 @@ static inline int qelr_configure_qp_sq(struct qelr_devctx *cxt,
 	qp->sq.max_wr++;	/* prod/cons method requires N+1 elements */
 	qp->wqe_wr_id = calloc(qp->sq.max_wr, sizeof(*qp->wqe_wr_id));
 	if (!qp->wqe_wr_id) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "create qp: failed shadow SQ memory allocation\n");
 		return -ENOMEM;
 	}
@@ -670,7 +670,7 @@ static inline int qelr_configure_qp_rq(struct qelr_devctx *cxt,
 		if (qp->rq.db_rec_map == MAP_FAILED) {
 			int errsv = errno;
 
-			DP_ERR(cxt->dbg_fp,
+			verbs_err(&cxt->ibv_ctx,
 			       "alloc context: doorbell rec mapping failed resp.db_rec_addr = %llx size=%d context->cmd_fd=%d errno=%d\n",
 			       resp->rq_db_rec_addr, cxt->kernel_page_size,
 			       cxt->ibv_ctx.context.cmd_fd, errsv);
@@ -688,7 +688,7 @@ static inline int qelr_configure_qp_rq(struct qelr_devctx *cxt,
 	qp->rq.max_wr++;	/* prod/cons method requires N+1 elements */
 	qp->rqe_wr_id = calloc(qp->rq.max_wr, sizeof(*qp->rqe_wr_id));
 	if (!qp->rqe_wr_id) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "create qp: failed shadow RQ memory allocation\n");
 		return -ENOMEM;
 	}
@@ -729,7 +729,7 @@ static inline int qelr_configure_qp(struct qelr_devctx *cxt, struct qelr_qp *qp,
 static inline void qelr_print_qp_init_attr(struct qelr_devctx *cxt,
 					   struct ibv_qp_init_attr_ex *attrx)
 {
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP,
+	verbs_debug(&cxt->ibv_ctx,
 		   "create qp: send_cq=%p, recv_cq=%p, srq=%p, max_inline_data=%d, max_recv_sge=%d, max_recv_wr=%d, max_send_sge=%d, max_send_wr=%d, qp_type=%d, sq_sig_all=%d\n",
 		   attrx->send_cq, attrx->recv_cq, attrx->srq,
 		   attrx->cap.max_inline_data, attrx->cap.max_recv_sge,
@@ -781,7 +781,7 @@ static inline void qelr_basic_qp_config(struct qelr_qp *qp,
 
 static void qelr_print_ah_attr(struct qelr_devctx *cxt, struct ibv_ah_attr *attr)
 {
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP,
+	verbs_debug(&cxt->ibv_ctx,
 		   "grh.dgid=[%#" PRIx64 ":%#" PRIx64 "], grh.flow_label=%d, grh.sgid_index=%d, grh.hop_limit=%d, grh.traffic_class=%d, dlid=%d, sl=%d, src_path_bits=%d, static_rate = %d, port_num=%d\n",
 		   be64toh(attr->grh.dgid.global.interface_id),
 		   be64toh(attr->grh.dgid.global.subnet_prefix),
@@ -793,7 +793,7 @@ static void qelr_print_ah_attr(struct qelr_devctx *cxt, struct ibv_ah_attr *attr
 
 static void qelr_print_qp_attr(struct qelr_devctx *cxt, struct ibv_qp_attr *attr)
 {
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP,
+	verbs_debug(&cxt->ibv_ctx,
 		   "\tqp_state=%d\tcur_qp_state=%d\tpath_mtu=%d\tpath_mig_state=%d\tqkey=%d\trq_psn=%d\tsq_psn=%d\tdest_qp_num=%d\tqp_access_flags=%d\tmax_inline_data=%d\tmax_recv_sge=%d\tmax_recv_wr=%d\tmax_send_sge=%d\tmax_send_wr=%d\tpkey_index=%d\talt_pkey_index=%d\ten_sqd_async_notify=%d\tsq_draining=%d\tmax_rd_atomic=%d\tmax_dest_rd_atomic=%d\tmin_rnr_timer=%d\tport_num=%d\ttimeout=%d\tretry_cnt=%d\trnr_retry=%d\talt_port_num=%d\talt_timeout=%d\n",
 		   attr->qp_state, attr->cur_qp_state, attr->path_mtu,
 		   attr->path_mig_state, attr->qkey, attr->rq_psn, attr->sq_psn,
@@ -818,7 +818,7 @@ int qelr_query_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 	struct qelr_devctx *cxt = get_qelr_ctx(qp->context);
 	int rc;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP, "QP Query %p, attr_mask=0x%x\n",
+	verbs_debug(&cxt->ibv_ctx, "QP Query %p, attr_mask=0x%x\n",
 		   get_qelr_qp(qp), attr_mask);
 
 	rc = ibv_cmd_query_qp(qp, attr, attr_mask,
@@ -997,24 +997,24 @@ int qelr_modify_qp(struct ibv_qp *ibqp, struct ibv_qp_attr *attr,
 	union ibv_gid sgid, *p_dgid;
 	int rc;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP, "QP Modify %p, attr_mask=0x%x\n",
+	verbs_debug(&cxt->ibv_ctx, "QP Modify %p, attr_mask=0x%x\n",
 		   qp, attr_mask);
 
 	qelr_print_qp_attr(cxt, attr);
 
 	rc = ibv_cmd_modify_qp(ibqp, attr, attr_mask, &cmd, sizeof(cmd));
 	if (rc) {
-		DP_ERR(cxt->dbg_fp, "QP Modify: Failed command. rc=%d\n", rc);
+		verbs_err(&cxt->ibv_ctx, "QP Modify: Failed command. rc=%d\n", rc);
 		return rc;
 	}
 
 	if (attr_mask & IBV_QP_STATE) {
 		rc = qelr_update_qp_state(qp, attr->qp_state);
-		DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP,
+		verbs_debug(&cxt->ibv_ctx,
 			   "QP Modify state %d->%d, rc=%d\n", qp->state,
 			   attr->qp_state, rc);
 		if (rc) {
-			DP_ERR(cxt->dbg_fp,
+			verbs_err(&cxt->ibv_ctx,
 			       "QP Modify: Failed to update state. rc=%d\n",
 			       rc);
 
@@ -1031,11 +1031,11 @@ int qelr_modify_qp(struct ibv_qp *ibqp, struct ibv_qp_attr *attr,
 			p_dgid = &attr->ah_attr.grh.dgid;
 			qp->edpm_disabled = !memcmp(&sgid, p_dgid,
 						    sizeof(sgid));
-			DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP,
+			verbs_debug(&cxt->ibv_ctx,
 				   "QP Modify: %p, edpm_disabled=%d\n", qp,
 				   qp->edpm_disabled);
 		} else  {
-			DP_ERR(cxt->dbg_fp,
+			verbs_err(&cxt->ibv_ctx,
 			       "QP Modify: Failed querying GID. rc=%d\n",
 			       rc);
 		}
@@ -1050,11 +1050,11 @@ int qelr_destroy_qp(struct ibv_qp *ibqp)
 	struct qelr_qp *qp = get_qelr_qp(ibqp);
 	int rc = 0;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP, "destroy qp: %p\n", qp);
+	verbs_debug(&cxt->ibv_ctx, "destroy qp: %p\n", qp);
 
 	rc = ibv_cmd_destroy_qp(ibqp);
 	if (rc) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "destroy qp: failed to destroy %p, got %d.\n", qp, rc);
 		return rc;
 	}
@@ -1068,8 +1068,7 @@ int qelr_destroy_qp(struct ibv_qp *ibqp)
 	if (qp->rq.db_rec_map)
 		munmap(qp->rq.db_rec_map, cxt->kernel_page_size);
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP,
-		   "destroy cq: successfully destroyed %p\n", qp);
+	verbs_debug(&cxt->ibv_ctx, "destroy cq: successfully destroyed %p\n", qp);
 
 	free(qp);
 
@@ -1498,7 +1497,7 @@ static inline int qelr_can_post_send(struct qelr_devctx *cxt,
 {
 	/* Invalid WR */
 	if (wr->num_sge > qp->sq.max_sges) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "error: WR is bad. Post send on QP %p failed\n",
 		       qp);
 		return -EINVAL;
@@ -1506,7 +1505,7 @@ static inline int qelr_can_post_send(struct qelr_devctx *cxt,
 
 	/* WR overflow */
 	if (qelr_wq_is_full(&qp->sq)) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "error: WQ is full. Post send on QP %p failed (this error appears only once)\n",
 		       qp);
 		return -ENOMEM;
@@ -1515,7 +1514,7 @@ static inline int qelr_can_post_send(struct qelr_devctx *cxt,
 	/* WQE overflow */
 	if (qelr_chain_get_elem_left_u32(&qp->sq.chain) <
 			QELR_MAX_SQ_WQE_SIZE) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "error: WQ PBL is full. Post send on QP %p failed (this error appears only once)\n",
 		       qp);
 		return -ENOMEM;
@@ -1524,13 +1523,13 @@ static inline int qelr_can_post_send(struct qelr_devctx *cxt,
 	if ((wr->opcode == IBV_WR_ATOMIC_CMP_AND_SWP ||
 	     wr->opcode == IBV_WR_ATOMIC_FETCH_AND_ADD) &&
 	    !qp->atomic_supported) {
-		DP_ERR(cxt->dbg_fp, "Atomic not supported on this machine\n");
+		verbs_err(&cxt->ibv_ctx, "Atomic not supported on this machine\n");
 		return -EINVAL;
 	}
 
 	if ((wr->send_flags & IBV_SEND_INLINE) &&
 	    (data_size > ROCE_REQ_MAX_INLINE_DATA_SIZE)) {
-		DP_ERR(cxt->dbg_fp, "Too much inline data in WR: %d\n", data_size);
+		verbs_err(&cxt->ibv_ctx, "Too much inline data in WR: %d\n", data_size);
 		return -EINVAL;
 	}
 
@@ -1786,7 +1785,7 @@ static int __qelr_post_send(struct qelr_devctx *cxt, struct qelr_qp *qp,
 		qp->prev_wqe_size = wqe->prev_wqe_size;
 
 		rc = -EINVAL;
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "Invalid opcode %d in work request on QP %p\n",
 		       wr->opcode, qp);
 		break;
@@ -1888,7 +1887,7 @@ int qelr_post_srq_recv(struct ibv_srq *ibsrq, struct ibv_recv_wr *wr,
 
 		if (!qelr_srq_elem_left(hw_srq) ||
 		    wr->num_sge > srq->hw_srq.max_sges) {
-			DP_ERR(cxt->dbg_fp,
+			verbs_err(&cxt->ibv_ctx,
 			       "Can't post WR  (%d,%d) || (%d > %d)\n",
 			       hw_srq->wr_prod_cnt, hw_srq->wr_cons_cnt,
 			       wr->num_sge,
@@ -1906,7 +1905,7 @@ int qelr_post_srq_recv(struct ibv_srq *ibsrq, struct ibv_recv_wr *wr,
 		hw_srq->wqe_prod++;
 		hw_srq->sge_prod++;
 
-		DP_VERBOSE(cxt->dbg_fp, QELR_MSG_SRQ,
+		verbs_debug(&cxt->ibv_ctx,
 			   "SRQ WR: SGEs: %d with wr_id[%d] = %" PRIx64 "\n",
 			    wr->num_sge, hw_srq->wqe_prod, wr->wr_id);
 
@@ -1917,7 +1916,7 @@ int qelr_post_srq_recv(struct ibv_srq *ibsrq, struct ibv_recv_wr *wr,
 			SRQ_SGE_SET(srq_sge, wr->sg_list[i].addr,
 				    wr->sg_list[i].length, wr->sg_list[i].lkey);
 
-			DP_VERBOSE(cxt->dbg_fp, QELR_MSG_SRQ,
+			verbs_debug(&cxt->ibv_ctx,
 				   "[%d]: len %d key %x addr %x:%x\n",
 				   i, srq_sge->length, srq_sge->l_key,
 				   srq_sge->addr.hi, srq_sge->addr.lo);
@@ -1939,7 +1938,7 @@ int qelr_post_srq_recv(struct ibv_srq *ibsrq, struct ibv_recv_wr *wr,
 		wr = wr->next;
 	}
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_SRQ,
+	verbs_debug(&cxt->ibv_ctx,
 		   "POST: Elements in SRQ: %d\n",
 		   qelr_chain_get_elem_left_u32(chain));
 	pthread_spin_unlock(&srq->lock);
@@ -1957,7 +1956,7 @@ int qelr_post_recv(struct ibv_qp *ibqp, struct ibv_recv_wr *wr,
 	uint8_t iwarp = IS_IWARP(ibqp->context->device);
 
 	if (unlikely(qp->srq)) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "QP is associated with SRQ, cannot post RQ buffers\n");
 		*bad_wr = wr;
 		return -EINVAL;
@@ -1976,7 +1975,7 @@ int qelr_post_recv(struct ibv_qp *ibqp, struct ibv_recv_wr *wr,
 
 		if (qelr_chain_get_elem_left_u32(&qp->rq.chain) <
 		    QELR_MAX_RQ_WQE_SIZE || wr->num_sge > qp->rq.max_sges) {
-			DP_ERR(cxt->dbg_fp,
+			verbs_err(&cxt->ibv_ctx,
 			       "Can't post WR  (%d < %d) || (%d > %d)\n",
 			       qelr_chain_get_elem_left_u32(&qp->rq.chain),
 			       QELR_MAX_RQ_WQE_SIZE, wr->num_sge,
@@ -2095,7 +2094,7 @@ static int process_req(struct qelr_qp *qp, struct qelr_cq *cq, int num_entries,
 		switch (wc->opcode) {
 		case IBV_WC_RDMA_WRITE:
 			wc->byte_len = qp->wqe_wr_id[qp->sq.cons].bytes_len;
-			DP_VERBOSE(cxt->dbg_fp, QELR_MSG_CQ,
+			verbs_debug(&cxt->ibv_ctx,
 				   "POLL REQ CQ: IBV_WC_RDMA_WRITE byte_len=%d\n",
 				   qp->wqe_wr_id[qp->sq.cons].bytes_len);
 			break;
@@ -2107,7 +2106,7 @@ static int process_req(struct qelr_qp *qp, struct qelr_cq *cq, int num_entries,
 		case IBV_WC_SEND:
 		case IBV_WC_BIND_MW:
 			wc->byte_len = qp->wqe_wr_id[qp->sq.cons].bytes_len;
-			DP_VERBOSE(cxt->dbg_fp, QELR_MSG_CQ,
+			verbs_debug(&cxt->ibv_ctx,
 				   "POLL REQ CQ: IBV_WC_RDMA_READ / IBV_WC_SEND\n");
 			break;
 		default:
@@ -2140,7 +2139,7 @@ static int qelr_poll_cq_req(struct qelr_qp *qp, struct qelr_cq *cq,
 				  IBV_WC_SUCCESS, 0);
 		break;
 	case RDMA_CQE_REQ_STS_WORK_REQUEST_FLUSHED_ERR:
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "Error: POLL CQ with ROCE_CQE_REQ_STS_WORK_REQUEST_FLUSHED_ERR. QP icid=0x%x\n",
 		       qp->sq.icid);
 		cnt = process_req(qp, cq, num_entries, wc, sq_cons,
@@ -2158,67 +2157,67 @@ static int qelr_poll_cq_req(struct qelr_qp *qp, struct qelr_cq *cq,
 
 			switch (req->status) {
 			case    RDMA_CQE_REQ_STS_BAD_RESPONSE_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_BAD_RESPONSE_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_BAD_RESP_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_LOCAL_LENGTH_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_LOCAL_LENGTH_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_LOC_LEN_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_LOCAL_QP_OPERATION_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_LOCAL_QP_OPERATION_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_LOC_QP_OP_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_LOCAL_PROTECTION_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_LOCAL_PROTECTION_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_LOC_PROT_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_MEMORY_MGT_OPERATION_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_MEMORY_MGT_OPERATION_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_MW_BIND_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_REMOTE_INVALID_REQUEST_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_REMOTE_INVALID_REQUEST_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_REM_INV_REQ_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_REMOTE_ACCESS_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_REMOTE_ACCESS_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_REM_ACCESS_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_REMOTE_OPERATION_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_REMOTE_OPERATION_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_REM_OP_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_RNR_NAK_RETRY_CNT_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "Error: POLL CQ with RDMA_CQE_REQ_STS_RNR_NAK_RETRY_CNT_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_RNR_RETRY_EXC_ERR;
 				break;
 			case    RDMA_CQE_REQ_STS_TRANSPORT_RETRY_CNT_ERR:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "RDMA_CQE_REQ_STS_TRANSPORT_RETRY_CNT_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_RETRY_EXC_ERR;
 				break;
 			default:
-				DP_ERR(cxt->dbg_fp,
+				verbs_err(&cxt->ibv_ctx,
 				       "IBV_WC_GENERAL_ERR. QP icid=0x%x\n",
 				       qp->sq.icid);
 				wc_status = IBV_WC_GENERAL_ERR;
@@ -2285,7 +2284,7 @@ static void __process_resp_one(struct qelr_devctx *cxt, struct qelr_cq *cq,
 			wc->wc_flags |= IBV_WC_WITH_INV;
 			break;
 		case QELR_RESP_RDMA:
-			DP_ERR(cxt->dbg_fp, "Invalid flags detected\n");
+			verbs_err(&cxt->ibv_ctx, "Invalid flags detected\n");
 			break;
 		default:
 			/* valid configuration, but nothing to do here */
@@ -2295,7 +2294,7 @@ static void __process_resp_one(struct qelr_devctx *cxt, struct qelr_cq *cq,
 		break;
 	default:
 		wc->status = IBV_WC_GENERAL_ERR;
-		DP_ERR(cxt->dbg_fp, "Invalid CQE status detected\n");
+		verbs_err(&cxt->ibv_ctx, "Invalid CQE status detected\n");
 	}
 
 	/* fill WC */
@@ -2451,7 +2450,7 @@ static struct qelr_srq *qelr_get_xrc_srq_from_cqe(struct qelr_cq *cq,
 	cxt = get_qelr_ctx(cq->ibv_cq.context);
 	srq = qelr_get_srq(cxt, srq_id);
 	if (unlikely(!srq)) {
-		DP_ERR(cxt->dbg_fp, "srq handle is null\n");
+		verbs_err(&cxt->ibv_ctx, "srq handle is null\n");
 		return NULL;
 	}
 
@@ -2479,7 +2478,7 @@ int qelr_poll_cq(struct ibv_cq *ibcq, int num_entries, struct ibv_wc *wc)
 		qp = cqe_get_qp(cqe);
 		if (!qp &&
 		    cqe_get_type(cqe) != RDMA_CQE_TYPE_RESPONDER_XRC_SRQ) {
-			DP_ERR(stderr,
+			verbs_err(verbs_get_ctx(qp->ibv_qp->context),
 			       "Error: CQE QP pointer is NULL. CQE=%p\n", cqe);
 			break;
 		}
@@ -2609,7 +2608,7 @@ struct ibv_xrcd *qelr_open_xrcd(struct ibv_context *context,
 	rc = ibv_cmd_open_xrcd(context, xrcd, sizeof(*xrcd), init_attr, &cmd,
 			       sizeof(cmd), &resp, sizeof(resp));
 	if (rc) {
-		DP_ERR(cxt->dbg_fp, "open xrcd: failed with rc=%d.\n", rc);
+		verbs_err(&cxt->ibv_ctx, "open xrcd: failed with rc=%d.\n", rc);
 		free(xrcd);
 		return NULL;
 	}
@@ -2625,7 +2624,7 @@ int qelr_close_xrcd(struct ibv_xrcd *ibxrcd)
 
 	rc = ibv_cmd_close_xrcd(xrcd);
 	if (rc) {
-		DP_ERR(cxt->dbg_fp, "close xrcd: failed with rc=%d.\n", rc);
+		verbs_err(&cxt->ibv_ctx, "close xrcd: failed with rc=%d.\n", rc);
 		free(xrcd);
 	}
 
@@ -2672,7 +2671,7 @@ qelr_create_xrc_srq(struct ibv_context *context,
 
 	cxt->srq_table[resp.srq_id] = srq;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_SRQ,
+	verbs_debug(&cxt->ibv_ctx,
 		   "create srq_ex: successfully created %p.\n", srq);
 
 	return ibv_srq;
@@ -2681,7 +2680,7 @@ err1:
 	qelr_destroy_srq_buffers(ibv_srq);
 	free(srq);
 err0:
-	DP_ERR(cxt->dbg_fp,
+	verbs_err(&cxt->ibv_ctx,
 	       "create srq: failed to create. rc=%d\n", rc);
 	return NULL;
 }
@@ -2707,7 +2706,7 @@ struct ibv_srq *qelr_create_srq_ex(struct ibv_context *context,
 	if (init_attr->srq_type == IBV_SRQT_XRC)
 		return qelr_create_xrc_srq(context, init_attr);
 
-	DP_ERR(cxt->dbg_fp, "failed to create srq type %d\n",
+	verbs_err(&cxt->ibv_ctx, "failed to create srq type %d\n",
 	       init_attr->srq_type);
 
 	return NULL;
@@ -2725,6 +2724,14 @@ static struct ibv_qp *create_qp(struct ibv_context *context,
 
 	qelr_print_qp_init_attr(cxt, attrx);
 
+#define QELR_CREATE_QP_SUPP_ATTR_MASK \
+	(IBV_QP_INIT_ATTR_PD | IBV_QP_INIT_ATTR_XRCD)
+
+	if (!check_comp_mask(attrx->comp_mask, QELR_CREATE_QP_SUPP_ATTR_MASK)) {
+		errno = EOPNOTSUPP;
+		return NULL;
+	}
+
 	qp = calloc(1, sizeof(*qp));
 	if (!qp)
 		return NULL;
@@ -2741,7 +2748,7 @@ static struct ibv_qp *create_qp(struct ibv_context *context,
 				  attrx, &req.ibv_cmd, sizeof(req),
 				  &resp.ibv_resp, sizeof(resp));
 	if (rc) {
-		DP_ERR(cxt->dbg_fp,
+		verbs_err(&cxt->ibv_ctx,
 		       "create qp: failed on ibv_cmd_create_qp with %d\n", rc);
 		goto err1;
 	}
@@ -2750,7 +2757,7 @@ static struct ibv_qp *create_qp(struct ibv_context *context,
 	if (rc)
 		goto err2;
 
-	DP_VERBOSE(cxt->dbg_fp, QELR_MSG_QP,
+	verbs_debug(&cxt->ibv_ctx,
 		   "create qp: successfully created %p. handle_hi=%x handle_lo=%x\n",
 		   qp, req.qp_handle_hi, req.qp_handle_lo);
 
@@ -2762,7 +2769,7 @@ static struct ibv_qp *create_qp(struct ibv_context *context,
 err2:
 	rc = ibv_cmd_destroy_qp(get_ibv_qp(qp));
 	if (rc)
-		DP_ERR(cxt->dbg_fp, "create qp: fatal fault. rc=%d\n", rc);
+		verbs_err(&cxt->ibv_ctx, "create qp: fatal fault. rc=%d\n", rc);
 err1:
 	if (qelr_qp_has_sq(qp))
 		qelr_chain_free(&qp->sq.chain);
