@@ -162,11 +162,6 @@ static int rxe_dealloc_mw(struct ibv_mw *ibmw)
 	return 0;
 }
 
-static int next_rkey(int rkey)
-{
-	return (rkey & 0xffffff00) | ((rkey + 1) & 0x000000ff);
-}
-
 static int rxe_post_send(struct ibv_qp *ibqp, struct ibv_send_wr *wr_list,
 			 struct ibv_send_wr **bad_wr);
 
@@ -191,7 +186,7 @@ static int rxe_bind_mw(struct ibv_qp *ibqp, struct ibv_mw *ibmw,
 	ibwr.send_flags = mw_bind->send_flags;
 	ibwr.bind_mw.bind_info = mw_bind->bind_info;
 	ibwr.bind_mw.mw = ibmw;
-	ibwr.bind_mw.rkey = next_rkey(ibmw->rkey);
+	ibwr.bind_mw.rkey = ibv_inc_rkey(ibmw->rkey);
 
 	ret = rxe_post_send(ibqp, &ibwr, &bad_wr);
 	if (ret)
@@ -266,15 +261,17 @@ static int cq_next_poll(struct ibv_cq_ex *current)
 {
 	struct rxe_cq *cq = container_of(current, struct rxe_cq, vcq.cq_ex);
 
-	advance_cq_cur_index(cq);
+	struct rxe_queue_buf *q = cq->queue;
+	uint32_t next_index = (cq->cur_index + 1) & q->index_mask;
 
-	if (check_cq_queue_empty(cq)) {
+	if (next_index == load_producer_index(q)) {
 		store_consumer_index(cq->queue, cq->cur_index);
 		pthread_spin_unlock(&cq->lock);
 		errno = ENOENT;
 		return errno;
 	}
 
+	cq->cur_index = next_index;
 	cq->wc = addr_from_index(cq->queue, cq->cur_index);
 	cq->vcq.cq_ex.status = cq->wc->status;
 	cq->vcq.cq_ex.wr_id = cq->wc->wr_id;
@@ -581,7 +578,7 @@ static struct ibv_srq *rxe_create_srq(struct ibv_pd *ibpd,
 	struct rxe_srq *srq;
 	struct ibv_srq *ibsrq;
 	struct ibv_create_srq cmd;
-	struct urxe_create_srq_resp resp = {};
+	struct urxe_create_srq_resp resp;
 	int ret;
 
 	srq = calloc(1, sizeof(*srq));
@@ -620,7 +617,7 @@ static struct ibv_srq *rxe_create_srq_ex(
 	struct rxe_srq *srq;
 	struct ibv_srq *ibsrq;
 	struct ibv_create_xsrq cmd;
-	struct urxe_create_srq_ex_resp resp = {};
+	struct urxe_create_srq_ex_resp resp;
 	int ret;
 
 	srq = calloc(1, sizeof(*srq));
